// 处理Java 类型的项目构建
var _ = require('lodash');
const execa = require('execa');
const replace = require('replace-in-file');
const fs = require('fs');
var sleep = require('sleep');
var pomParser = require("pom-parser");

/**
 * 构建Java项目
 * @param {*} projectObj 项目信息
 * @param {*} envObj 环境信息
 * @param {*} typeObj 构建源数据
 * @param {*} configList 配置信息
 * @param {*} branch 分支
 * @param {*} version 版本号
 */
const buildJava = async (projectObj, envObj, typeObj, configList, branch, version) => {
    //start to buid 
    // 获取项目源数据
    var projectMetaData = JSON.parse(projectObj.meta_data);
    console.log(JSON.stringify(projectMetaData));

    // 下载到的地址
    var source_git_url = projectObj.source_git_url;
    // 项目名
    var project_name = envObj.env_id + "-" + projectObj.name;
    // 项目文件夹
    var project_dir = __dirname + '/../tmp/' + project_name;
    // gitlab操作令牌
    var token = envObj.target_git_token;
    // 目标gitlab地址
    var target_git_url = envObj.target_git_url;
    // 目标gitlab用户名
    var target_git_name = envObj.target_git_name;
    // 仓库分组地址
    var registry_group = envObj.registry_group;

    var registry_url = envObj.registry_url;

    // 删除原项目
    let { stdout1 } = await execa('rm', ['-rf', project_dir]);
    console.log(JSON.stringify("删除文件夹" + project_dir));
    //下载git项目
    if (_.isEmpty(branch)) {
        branch = "master";
    }
    let { stdout2 } = await execa(`git`, ['clone', '-b', branch, source_git_url, project_name], { cwd: __dirname + '/../tmp/' })
    console.log("下载项目")

    // 删除.git 等文件
    let { stdout3 } = await execa('rm', ['-rf', project_dir + '/.git']);
    console.log("删除.git");

    //gitlab 创建项目
    let stdout4 = await execa('curl', [
        '--header',
        `PRIVATE-TOKEN:${token}`,
        '-X',
        'DELETE',
        `${target_git_url}/api/v4/projects/{${target_git_name}}%2f${project_name}`
    ]);
    // console.log(stdout4);

    // sleep
    sleep.sleep(3);

    let stdout5 = await execa('curl', [
        '--header',
        `PRIVATE-TOKEN:${token}`,
        '-X',
        'POST',
        `${target_git_url}/api/v4/projects?name=${project_name}`
    ]);

    // console.log(stdout5);
    var git_info = JSON.parse(stdout5.stdout);
    //获取项目远程地址
    var git_remote = git_info.http_url_to_repo;
    console.log("git项目的远程地址为:");
    console.log(git_remote);

    //配置文件替换
    if (!_.isEmpty(configList)) {
        configList.forEach((config) => {
            (async () => {
                let configType = config.type;
                switch (configType) {
                    case 1:
                        // 替换模式
                        var key_value = JSON.parse(config.content);
                        await replaceFileByMap(key_value, `${project_dir}/${config.path}`, config.content)
                        break;
                    case 2:
                        // 覆盖模式
                        fs.writeFileSync(`${project_dir}/${config.path}`, config.content);
                        break;
                    default:
                        console.warn("Unknow Config type");
                }
            })();
        })
    }

    // 先看看build语句
    // 默认跳过测试
    var build_script = _.get(projectMetaData, "build_script", "mvn clean install package -Dmaven.test.skip=true");
    // 识别要扫描的模块 
    var module_list = _.get(projectMetaData, "module_list", ["."]);
    // 初始化变量
    var variables_str = "";
    var cache = "";
    var docker_build_str = "";
    var docker_tag_str = "";
    var docker_push_str = "";

    // todo 处理docker登录问题
    var username = _.get(envObj, 'registry_username');
    var password = _.get(envObj, 'registry_password');
    // docker login ${IMAGE_HOST} -u yzy -p 123qweQWE@ 

    if (!_.isEmpty(username) && !_.isEmpty(password)) {
        docker_build_str += "docker login ${IMAGE_HOST} -u " + username + " -p " + password;
    }


    // 关键替换
    for (let v of module_list) {
        let opt = { filePath: project_dir + "/" + v + "/pom.xml" }
        var pom_object = await pomParserPromise(opt);
        // console.log(JSON.stringify(pom_object));
        // 这里默认情况下使用通配符
        var executeFIle = "*.jar";

        var service_name = pom_object.project.name;
        var dockerfile_name = "dockerfile_" + service_name;

        variables_str += "  " + "IMAGE_NAME-" + service_name + ': "${IMAGE_HOST}/' + registry_group + '/${CI_PROJECT_NAME}/' + _.lowerCase(service_name) + '"' + "\n";
        variables_str += "  " + 'IMAGE_VERSION-' + service_name + ': "${IMAGE_HOST}/' + registry_group + '/${CI_PROJECT_NAME}/' + _.lowerCase(service_name) + ':${CI_COMMIT_TAG}"' + "\n";
        variables_str += "  " + 'IMAGE_LATEST-' + service_name + ': "${IMAGE_HOST}/' + registry_group + '/${CI_PROJECT_NAME}/' + _.lowerCase(service_name) + ':latest"' + "\n";
        //缓存地址
        cache += "    - " + v + "/target/\n";
        docker_build_str += "    " + "- docker build -t ${IMAGE_VERSION-" + service_name + "} -f " + dockerfile_name + " ." + "\n";
        docker_tag_str += "    " + '- docker tag ${IMAGE_VERSION-' + service_name + '} ${IMAGE_LATEST' + service_name + '}' + "\n";
        docker_push_str += "    " + '- docker push ${IMAGE_VERSION-' + service_name + '}' + "\n";
        docker_push_str += "    " + '- docker push ${IMAGE_LATEST-' + service_name + '}' + "\n";
        // 这里要写dockerfile
    }

    // 创建gitlab-ci
    fs.writeFileSync(`${project_dir}/.gitlab-ci.yml`, typeObj.gitlab_ci);
    var gitlab_ci_map = {
        "${registry_url}": registry_url,
        "{variables_str}": variables_str,
        "{cache}": cache,
        "{docker_build_str}": _.trimEnd(docker_build_str, "\n"),
        "{docker_tag_str}": _.trimEnd(docker_tag_str, "\n"),
        "{docker_push_str}": _.trimEnd(docker_push_str, "\n"),
        "{build_str_java}": build_script
    };
    await replaceFileByMap(gitlab_ci_map, `${project_dir}/.gitlab-ci.yml`);
    // 提供gitlab-ci

    // 这个是循环的！！！！
    // 创建dockerfile
    // fs.writeFileSync(`${project_dir}/dockerfile`, typeObj.docker_file);
    // 替换dokerfile


    // todo 创建docker-compose
    // 替换docker-compose

    // 提交上传代码
    // 打上tag
}

/**
 * 按照key-value的map 替换一组值
 * @param {*} map  替换字符串对
 * @param {*} patch  文件路径
 */
const replaceFileByMap = async (map, patch) => {
    const quote = function (str) {
        return (str + '').replace(/[.?*+^$[\]\\(){}|-]/g, "\\$&");
    };
    if (!_.isEmpty(map)) {
        var from = [];
        var to = [];
        _.forEach(map, function (value, key) {
            (async () => {
                let quoteKey = quote(key);
                // console.log(quoteKey);
                from.push(eval(`/${quoteKey}/g`));
                to.push(value);
            })();
        });
        let options = {
            files: patch,
            from: from,
            to: to,
        };
        let loga = await replace(options);
        console.log(loga);
    }
}


/**
 * promise 版的解析Pom文件
 * @param {*} opt 
 */
var pomParserPromise = (opts) => {
    return new Promise((reslove, reject) => {
        pomParser.parse(opts, function (err, pomResponse) {
            if (err) {
                console.log("ERROR: " + err);
                reject(err);
            }
            reslove(pomResponse.pomObject);
        });
    });
}

module.exports = {
    buildJava,
}